


def from_pretty_yaml(pretty_yaml: str) -> str:
    """Convert pretty yaml to yaml."""

    # removes leading whitespace so that one can write the first yaml
    # line in the test on the line below the triple quotes
    return pretty_yaml.lstrip()



from dataclasses import dataclass
from enum import StrEnum


@dataclass
class Metadata:
    name: str


class ModelKind(StrEnum):
    VOLUME = "AvatarVolume"
    SCHEMA = "AvatarSchema"
    PARAMETERS = "AvatarParameters"
    PRIVACY_METRICS = "AvatarPrivacyMetricsParameters"
    SIGNAL_METRICS = "AvatarSignalMetricsParameters"
    REPORT = "AvatarReportParameters"
    ADVICE = "AvatarAdviceParameters"



from __future__ import annotations

from dataclasses import asdict
from enum import StrEnum

import yaml

DEFAULT_INDENT_WIDTH = 2

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from _typeshed import DataclassInstance

# Allows StrEnum to be serialized as a string
yaml.SafeDumper.add_multi_representer(
    StrEnum,
    yaml.representer.SafeRepresenter.represent_str,
)


def to_yaml(obj: DataclassInstance) -> str:
    """Convert an object to yaml."""
    return yaml.safe_dump(asdict(obj), indent=DEFAULT_INDENT_WIDTH, sort_keys=False)



from avatar_yaml.models.common import Metadata
import pytest

from avatar_yaml.models.schema import (
    TableDataInfo,
    TableInfo,
    get_standard_schema,
    Schema,
    ModelKind,
    SchemaSpec,
    get_standard_schema,
    _get_standard_schema,
)
from tests.conftest import from_pretty_yaml


def test_get_standard_schema_private():
    expected = Schema(
        kind=ModelKind.SCHEMA,
        metadata=Metadata(name="test"),
        spec=SchemaSpec(
            tables=[
                TableInfo(
                    name="table",
                    data=TableDataInfo(volume="volume", file="file.csv"),
                )
            ],
        ),
    )

    schema = _get_standard_schema(
        name="test", table_name="table", volume="volume", file="file.csv"
    )

    assert schema == expected


def test_get_standard_schema():
    schema = get_standard_schema(name="test", table_name="table", volume="volume", file="file.csv")
    expected = from_pretty_yaml("""kind: AvatarSchema
metadata:
  name: test
spec:
  tables:
  - name: table
    data:
      volume: volume
      file: file.csv
""")
    assert schema == expected



import pytest
from avatar_yaml.models.common import Metadata, ModelKind
from avatar_yaml.models.volume import Volume, VolumeSpec, get_volume
from tests.conftest import from_pretty_yaml


@pytest.fixture
def volume():
    return Volume(
        kind=ModelKind.VOLUME,
        metadata=Metadata(name="test_metadata"),
        spec=VolumeSpec(url="http://example.com"),
    )


def test_volume_to_yaml(volume: Volume):
    expected_yaml = from_pretty_yaml("""
kind: AvatarVolume
metadata:
  name: test_metadata
spec:
  url: http://example.com
""")
    output = get_volume(name="test_metadata", url="http://example.com")
    assert output == expected_yaml



from dataclasses import dataclass

from avatar_yaml.models.common import Metadata, ModelKind
from avatar_yaml.yaml_utils import to_yaml


@dataclass(frozen=True)
class VolumeSpec:
    url: str


@dataclass(frozen=True)
class Volume:
    kind: ModelKind
    metadata: Metadata
    spec: VolumeSpec


def get_volume(name: str, url: str) -> str:
    """Get the yaml from a volume."""
    volume = Volume(
        kind=ModelKind.VOLUME,
        metadata=Metadata(name=name),
        spec=VolumeSpec(url=url),
    )

    return to_yaml(volume)



from dataclasses import dataclass
from enum import StrEnum

from avatar_yaml.models.common import Metadata, ModelKind
from avatar_yaml.yaml_utils import to_yaml


class ColumnType(StrEnum):
    INT = "int"
    BOOL = "bool"
    CATEGORY = "category"
    NUMERIC = "float"
    DATETIME = "datetime"


@dataclass(frozen=True)
class TableDataInfo:
    volume: str
    file: str


@dataclass(frozen=True)
class ColumnInfo:
    field: str
    type: ColumnType
    value_type: str | None
    identifier: bool = False
    primary_key: bool = False
    time_series_time: bool = False


@dataclass(frozen=True)
class TableInfo:
    name: str
    data: TableDataInfo


@dataclass(frozen=True)
class SchemaSpec:
    tables: list[TableInfo]


@dataclass(frozen=True)
class Schema:
    kind: ModelKind
    metadata: Metadata
    spec: SchemaSpec


def get_table_data(volume: str, file: str) -> TableDataInfo:
    return TableDataInfo(volume=volume, file=file)


def get_original_table(table_name: str, volume: str, file: str) -> TableInfo:
    return TableInfo(
        name=table_name,
        data=get_table_data(volume=volume, file=file),
    )


def _get_standard_schema(name: str, table_name: str, volume: str, file: str) -> Schema:
    return Schema(
        kind=ModelKind.SCHEMA,
        metadata=Metadata(name=name),
        spec=SchemaSpec(
            tables=[get_original_table(table_name=table_name, volume=volume, file=file)],
        ),
    )


def get_standard_schema(name: str, table_name: str, volume: str, file: str) -> str:
    return to_yaml(_get_standard_schema(name, table_name, volume, file))
